#!/usr/bin/env python

import urllib.request
from bs4 import BeautifulSoup
from bs4.element import Comment

__all__ = [] #utilisé pour n'autoriser l'accès à aucune fonctions

def tag_visible(element):
    if element.parent.name in ['style', 'script', 'head', 'title', 'meta', '[document]']: # si la balise mère est l'une d'elle
        return False                                                                            #retourner False
    if isinstance(element, Comment): #si l'élément est un commentaire
        return False                    #retourner False
    return True                 #sinon retourner True

def html_to_text(data):
    soup = BeautifulSoup(data, 'html.parser') #parse le html
    texts = soup.findAll(text=True) #garde seulement le texte
    visible_texts = filter(tag_visible, texts) #le texte visible
    return u" ".join(t.strip() for t in visible_texts)
    #ajoute le contenu de t (unicode) avec un espace en séparation

def tri_char(texte):
    count= 0
    counter=[]
    liste_char = []
    for ch in texte:                #Pour chaques caractères
        if ch in liste_char:            #Si le caractère est contenu dans la liste de caractères
            pos = liste_char.index(ch)      #obtient l'index du caractère dans la liste de caractères
            counter[pos].append(count)      #ajoute la position du caractère dans la liste des positions
        else:                           #Si le caractère n'existe pas dans la liste
            liste_char.append(ch)           #Ajout du crarctère dans la liste
            counter.append([count])         #Ajout de la position du caractère
        count+=1
    return (liste_char, counter)    #retourn liste de caractères et liste d'occurences

def getsite(url):
    #ouvre et accède au dernier stade de chargement de la page avant de lire son contenu
    html = urllib.request.urlopen(url).read()
    text = html_to_text(html) #parse la page
    return text #retourne le texte



#!/usr/bin/env python
__all__ = [] #utilisé pour n'autoriser l'accès à aucune fonctions

def GetAlphabet(AlphaList, OccList, threshold):
#demande la liste des caractères trouvés sur la pages et les occurences de ces mêmes caractères
    alphabet = []
    base = ''
    counter = 0
    #initialise les varables utiles

    for i in AlphaList:
        if len(OccList[counter]) >= threshold: #si le caractère apparait plus de n fois sur la page web
            alphabet.append(i)          #on ajoute le caractère dans le dictionnaire de caractères
        counter +=1
    return alphabet
    # retourne un nouvel alphabet contenant tout les caractères ayant au moins n occurences avec n=threshold

def IntToBase(Number, AlphabetList, OccurList, threshold=10):
    alphabet = GetAlphabet(AlphabetList, OccurList, threshold)
    b_nbr = len(alphabet) - 1
    base, out = [], ''
    delimiter = str(alphabet[-1])

    for trame in Number:
        n = int(trame)
        while n > 0:
            n, r = divmod(n,b_nbr)    # divise le nombre par la base et met le reste dans i
            base.append(alphabet[r])           # ajoute a la base le caractère correspondant au code i
        out += ''.join(base[::-1]) + delimiter
        base=[]
#génère une base alphanumérique afin de réduire la taille des fichier et de réduire les répétitions d'occurences
    return out[:-1]

def BaseToInt(base, AlphabetList, OccurList, threshold=10): #inverse de hex to base
    alphabet = GetAlphabet(AlphabetList, OccurList, threshold)
    b_nbr = len(alphabet) - 1
    delimiter = str(alphabet[-1])
    fin = []
    for trame in base.split(delimiter):
        out = 0
        for i in trame: # on multiplie la sortie par la base en ajoutant la valeur correspondant au caractère
            out = b_nbr * out + alphabet.index(i)
        fin.append(out)
    return fin



#!/usr/bin/env python

import binascii         #https://docs.python.org/3/library/binascii.html
import hashlib          #https://docs.python.org/fr/3.6/library/hashlib.html
import lzma             #utilitaire de compression de données basé sur l'algorithme LZMA

from NimingCypher.BaseConverter import IntToBase, BaseToInt
from secrets import choice

__all__ = []    #utilisé pour n'autoriser l'accès à aucune fonctions
threshold = 10  #utilisé pour définir le nb d'occurrence minimale nécessaire pour l'utilisation dans la base
chunk_size = 64

def intersperse(lst, item):
    result = [item] * (len(lst) * 2 - 1)
    result[0::2] = lst
    return result

def chiffrer(Clear_tx, listes_globales=[]):
    if listes_globales==[]:
        Clear_tx, listes_globales = Clear_tx

    liste_char, liste_pos = listes_globales


    good_ch=[hex(int(choice(liste_pos[liste_char.index(ch)])))[2:] for ch in Clear_tx if ch in liste_char]

    bad_ch=len(Clear_tx)-len(good_ch)

    final_gen=intersperse(good_ch,' ')

    """ANCIENNE METHODE - On passe de 10s à 2.2s avec le nv code"""
##    final_tx = ""
##    not_found = 0
##    for ch in Clear_tx: #pour chaque caractères
##        if ch in liste_char: #si le carctère est dans la liste des caractères
##            idx = liste_char.index(ch)  #on trouve son index dans la liste
##            #on récupère la liste de toutes les occurences de ce caractère
##            #qui possède le même index que la première liste
##            lpos = liste_pos[idx]
##
##            #on choisit une de ces occurence aléatoirement
##            position = int(lpos[(rnd(0,len(lpos)-1))])
##
##            #on converti la position en hexa et on l'ajoute
##            final_tx += str(hex(position))[2:] + " "
##        else: #si le caractère n'existe pas dans la liste
##            not_found += 1
##            final_tx += "0000 "
##    print(str(not_found) + ' caractères non trouvés !')
##    return(final_tx[:-1]) #retourne tout sauf le dernier caractère " "
    """FIN ANCIENNE METHODE"""
    print('%s caractères non trouvés !' % (bad_ch))
    return ''.join(final_gen)

def dechiffrer(Encrypted_tx, TextFromWeb):
    OutText = []
    for ch in Encrypted_tx.split(' '): #pour chaque bloc hexa
        if ch != '': #vérifie que le bloc soit bien existant
            try:
                OutText.append(str(TextFromWeb[int(ch,16)])) #retourne le caractère qui correspond à la position en hexa
            except:
                print("Erreur lors du déchiffrement !")
                OutText.append('?')
    return ''.join(OutText)

def GenSum(FileBytes): #génère un hash en sha256 pour la vérification des fichiers appelé sum
    return hashlib.sha256(FileBytes).hexdigest()

def FileToBase(chemin, alphabet, occurlist):
    file = open(chemin,"rb") # r --> read ; b --> byte
    binary_data = file.read() #stock le fichier en bytes dans byte_file
    file.close()
    len_bin = len(binary_data)
    #on transforme les bytes en tableau d'integer de taille chunk_size
    splitted_bin=[int.from_bytes(binary_data[ct:ct+chunk_size], byteorder='little') for ct in range(0, len_bin, chunk_size)]
    #On converti le tableau d'integer en base personnalisée afin d'éconmiser de l'espace
    base = IntToBase(splitted_bin, alphabet, occurlist, threshold)
    return base

def BaseToFile(base_data , path, alphabet, occurlist):#inverse de FileToBase
    try:
        file = open(path,"wb+") # w --> write ; b --> byte ; + --> créer le fichier si inexistant
        int_data = BaseToInt(base_data, alphabet, occurlist, threshold) #base perso vers hexa
        #on écrit le tableau d'integer converti en bytes dans le fichier en supprimant les bytes nuls en fin de ligne
        file.write(b''.join([int(i).to_bytes(chunk_size, byteorder='little') for i in int_data]).rstrip(b'\x00'))
        file.close()
        return True     #aucune erreur
    except:
        return False    #une erreur est survenue

def Compress(data): #compression des données
    comp_data = lzma.compress(data)
    return comp_data

def Decompress(data): #décompression des données
    decomp_data = lzma.decompress(data)
    return decomp_data

def ChiffrFile(location, listes): #chiffre le fichier
    liste_char, liste_pos = listes #liste des caractères présents sur le site et toutes les occurences
    base = FileToBase(location, liste_char, liste_pos) #lis le fichier et le converti en base personalisée
    chfr = chiffrer(base, listes) #on chiffre la base (le texte)
    compr_tx = Compress(chfr.encode()) #on compresse le texte chiffré
    return compr_tx

def DeChiffrFile(textencrypted, textfromweb, listes, path): #dechiffre le fichier
    liste_char,liste_pos = listes #liste des caractères présents sur le site et toutes les occurences
    decompress_tx = Decompress(textencrypted).decode()      #on décompresse et converti les bytes en texte
    dechTx = dechiffrer(decompress_tx, textfromweb)         #on déchiffre
    return BaseToFile(dechTx, path, liste_char, liste_pos)  #on retourne la base transformée en texte



#!/usr/bin/env python

##    Usage:
##
##    >>> from NimingCypher import NCrypter
##    >>> crypter = NCrypter("https://key.com")
##    >>> encrypted_str = crypter.crypt_text("simple string")
##    >>> print(encrypted_str)

from NimingCypher.Chiffrement import ChiffrFile, DeChiffrFile, Compress, Decompress, GenSum, dechiffrer, chiffrer
from NimingCypher.GetWebText import getsite, tri_char

#Toutes les explications sont dans les fichiers correspondants
#==================================================================================================================================
class NCrypter:

    def __init__(self, clef):
        self.result = self.setkey(clef)

    def setkey(self, clef):
        self._key = clef #_key pour rendre le paramètre 'key' privé
        try:
            self.textfromweb = getsite(self._key)
            self.charlist = tri_char(self.textfromweb)
            #Initialisation terminée !
            return True
        except:
            #Erreur de connexion !
            return False

    def crypt_file(self, path):
        file = open(path,"rb")
        filebytes = file.read()
        file.close()
        Sum = GenSum(filebytes)
        del filebytes
        fl = ChiffrFile(path, self.charlist)
        return (Sum, fl) #hash sha256 et le fichier chiffré

    def decrypt_file(self, textencrypte, path, open_end=False):
        from os import startfile
        if DeChiffrFile(textencrypte, self.textfromweb, self.charlist, path):
            if open_end == True:
                startfile(path)     #ouvre le fichier
            file = open(path,"rb")    #pour le sum
            filebytes = file.read()
            Sum = GenSum(filebytes)
            file.close()
            del filebytes
            return Sum
        else:
            return "Erreur lors du déchiffrement !"

    def crypt_text(self, text):
        return chiffrer(text,self.charlist)

    def decrypt_text(self, text):
        return dechiffrer(text, self.textfromweb)

#==================================================================================================================================



#!/usr/bin/env python

##    Ce module permet de chiffrer des chaines de caractères et des fichier.
##    Il permet une grande sécurité, et rapidité pour les
##    conversations instantanées.

__all__ = ["NCrypter"]
from NimingCypher.Niming import NCrypter



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='NimingCypher',
    version="2.0.3",
    description='High security encryption module for instant messaging',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Arnaud Gissinger',
    license='MIT',
    classifiers=[
        #https://pypi.python.org/pypi?%3Aaction=list_classifiers
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Topic :: Security :: Cryptography',

        # Pick your license as you wish (should match "license" above)
         'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        ],
    keywords='encryption module instant messaging free',
    install_requires=['beautifulsoup4'],
    packages=find_packages(),
)
